from __future__ import unicode_literals

import logging
import os

from mopidy import config, ext


__version__ = 'v0.1.1'


logger = logging.getLogger(__name__)


class Extension(ext.Extension):

    dist_name = 'Mopidy-RNZ'
    ext_name = 'rnz'
    version = __version__

    def get_default_config(self):
        conf_file = os.path.join(os.path.dirname(__file__), 'ext.conf')
        return config.read(conf_file)

    def get_config_schema(self):
        schema = super(Extension, self).get_config_schema()
        schema['http_cache'] = config.String()
        return schema

    def setup(self, registry):
        from .backend import RNZBackend
        registry.add('backend', RNZBackend)



from __future__ import unicode_literals

import logging
import os
import pykka
import requests
import requests_cache
from dateutil.parser import parse as parse_date
from mopidy import backend, httpclient
from mopidy.models import Ref, Artist, Album, Track

import mopidy_rnz

try:
    import xml.etree.cElementTree as ET
except ImportError:
    import xml.etree.ElementTree as ET

logger = logging.getLogger(__name__)

from mopidy_rnz import content


class RNZBackend(pykka.ThreadingActor, backend.Backend):
    def __init__(self, config, audio):
        super(RNZBackend, self).__init__()
        self.library = RNZLibraryProvider(backend=self)
        self.uri_schemes = ['rnz']

        http_cache = config['rnz']['http_cache']
        http_cache = os.path.expanduser(http_cache)
        logging.info("http_cache: %s", http_cache)
        requests_cache.install_cache(http_cache, backend='sqlite', expire_after=300)

        proxy_config = config['proxy']

        self.session = requests.Session()
        if proxy_config is not None:
            proxy = httpclient.format_proxy(proxy_config)
            self.session.proxies.update({'http': proxy, 'https': proxy})

        full_user_agent = httpclient.format_user_agent("%s/%s" % (
            mopidy_rnz.Extension.dist_name,
            mopidy_rnz.__version__))

        logging.debug('user_agent: %s', full_user_agent)

        self.session.headers.update({'user-agent': full_user_agent})

        # def on_start(self):
        # self.library.get_podcasts()

    def download(self, url):
        return self.session.get(url)


def _duration(s):
    s = s.split(':')
    duration = int(s[-1])
    i = len(s)
    if i > 1:
        duration += 60 * int(s[-2])
    if i > 2:
        duration += 60 * 60 * int(s[-3])
    return duration



class RNZLibraryProvider(backend.LibraryProvider):
    root_directory = Ref.directory(uri='rnz:root', name='RNZ')
    PODCASTS_URI = 'https://h1.danbrough.org/data/podcastinfo_v1.json'
    NAMESPACES = {'itunes': 'http://www.itunes.com/dtds/podcast-1.0.dtd'}
    podcast_items = {}

    def browse(self, uri):
        logger.debug("browse() %s for backend: %s", uri, self.backend)
        result = []

        if not uri.startswith('rnz:'):
            return result

        if uri == 'rnz:root':
            result.append(Ref.track(name='Latest News Bulletin', uri='rnz:news'))
            result.append(Ref.directory(name='Streams', uri='rnz:streams'))
            result.append(Ref.directory(name='Podcasts', uri='rnz:podcasts'))
            return result

        if uri == 'rnz:streams':
            return [
                Ref.track(
                    name=stream.name,
                    uri='rnz:stream:%s' % stream.name
                ) for stream in content.streams
            ]

        if uri == 'rnz:podcasts':
            return [
                Ref.directory(
                    name=podcast['title'],
                    uri='rnz:podcast:%s' % podcast['title']
                ) for podcast in self.get_podcasts()
            ]

        if uri.startswith('rnz:podcast:'):
            title = uri[12:]
            podcast = self.podcasts_map[title]
            podcast_url = podcast['urls']
            r = self.download(podcast_url)

            if r.status_code != 200:
                logging.error("failed to download %s", podcast_url)
                return None

            tree = ET.fromstring(r.text.encode('utf-8'))

            album = Album(
                artists=[Artist(name='RNZ')],
                images=[podcast['imageURL']],
                name=podcast['title'],
            )

            for item in tree.iter('item'):
                title = item.find('title').text.strip()
                logger.debug("got title %s", title)

                duration = item.find('itunes:duration', self.NAMESPACES)

                if duration is not None:
                    logger.debug("got duration %s", duration.text.strip())

                track_url = item.find('enclosure').get('url')

                result.append(Ref.track(
                    name=title,
                    uri='rnz:podcast_item:%s' % track_url,
                ))

                track_date = item.find('pubDate').text.strip()
                logger.debug("track_date: %s",track_date)
                track_date = parse_date(track_date).strftime('%Y-%m-%d')

                self.podcast_items[track_url] = Track(
                    name=title,
                    album=album,
                    artists=[Artist(name='RNZ')],
                    uri=track_url,
                    comment=item.find('description').text.strip(),
                    date=track_date,
                    length=_duration(
                        item.find('itunes:duration', self.NAMESPACES).text.strip()) * 1000
                )
            return result

        return []

    # def get_images(self, uris):
    #     logger.info('get_images(): %s',uris)
    #     # if len(uris) == 0 and uris[0] == 'rnz:news':
    #     #     pass
    #
    #     return super(RNZLibraryProvider,self).get_images(uris)

    def lookup(self, uri):
        logger.debug("lookup() %s", uri)
        result = []

        if not uri.startswith('rnz:'):
            return result

        if uri == 'rnz:news':
            from .news import get_news_info
            title, url, duration = get_news_info(self.download)
            return [
                content.news_track.replace(uri=url).replace(name=title).replace(length=duration)]

        if uri == 'rnz:streams':
            return content.streams

        if uri.startswith('rnz:stream:'):
            return [content.stream_map[uri[11:]]]

        if uri.startswith('rnz:podcast:'):
            return [self.podcasts_map[uri[12:]]]

        if uri.startswith('rnz:podcast_item:'):
            return [self.podcast_items[uri[17:]]]

        return result

    def download(self, url):
        logger.info("RNZLibraryProvider::download() url:%s", url)
        return self.backend.download(url)

    def get_podcasts(self):
        r = self.download(RNZLibraryProvider.PODCASTS_URI)
        if r.status_code != 200:
            logger.error("RNZ: Failed to download %s", RNZLibraryProvider.PODCASTS_URI)
            return []
        podcasts = r.json()
        for podcast in podcasts:
            title = podcast['title']
            if title.startswith('RNZ: '): podcast['title'] = title[4:].strip()
        self.podcasts = sorted(podcasts, key=lambda x: x['title'])
        self.podcasts_map = {podcast['title']: podcast for podcast in self.podcasts}
        logger.info("RNZ: Discovered %d podcasts", len(podcasts))
        return self.podcasts




from __future__ import unicode_literals,print_function

import hashlib
import logging
import re

logger = logging.getLogger(__name__)

"""
RNZ Magical function
"""


def prog_url(id):
    s = b"%dq6kzN3TQ29ubhUUhdOcirS0fiNITkMMLR5HyU5Sv" % id
    s = hashlib.sha1(s).digest().encode('hex')
    return "http://www.rnz.co.nz/audio/pdata/%s%s.json" % (s, hex(id)[2:])


_cleanr = re.compile('<.*?>')


"""
Returns the audio url for the latest RNZ news bulletin
"""


def get_news_info(download_func):
    r = download_func('http://www.radionz.co.nz/news')

    if r.status_code != 200:
        logger.error('Failed to download %s', 'http://www.radionz.co.nz/news')
        return None

    content = r.text
    content = content[content.find('Latest bulletin'):][:100]
    content = content[content.find('X') + 1:]
    content = content[:content.find('"')]
    code = int(content)
    url = prog_url(code)

    r = download_func(url)

    if r.status_code != 200:
        logger.error("Failed to download: %s", url)
        return None

    data = r.json()
    audio_url = data['item']['audio']['mp3']['url']
    audio_title = data['item']['body']
    audio_title = re.sub(_cleanr, '', audio_title)
    duration = 0
    s = data['item']['duration']
    i = s.find(u'\u2032')
    if i > -1:
        duration = 60*int(s[:i])
        s = s[i+1:]
    duration += int(s[:-1])



    return audio_title,audio_url,duration*1000


if __name__ == '__main__':
    import requests
    print("news url:", get_news_info(requests.get))



from mopidy.models import Album, Artist, Ref, Track,Image
import logging

logger = logging.getLogger(__name__)


_rnz_artist = Artist(name='RNZ')

_national_album = Album(
  artists = [_rnz_artist],
  images = ['http://www.radionz.co.nz/brand-images/rnz-national.jpg'],
  name = 'RNZ National',
)

_concert_album = Album(
  artists = [_rnz_artist],
  images = ['http://www.radionz.co.nz/brand-images/rnz-concert.jpg'],
  name = 'RNZ Concert',
)

_news_album = Album(
  artists = [_rnz_artist],
  images = ['http://www.radionz.co.nz/brand-images/rnz-news.jpg'],
  name = 'RNZ News',
)

_parliament_album = Album(
  artists = [_rnz_artist],
  images = ['http://www.radionz.co.nz/brand-images/rnz-parliament.jpg'],
  name = 'RNZ Parliament',
)

_international_album = Album(
  artists = [_rnz_artist],
  images = ['http://www.radionz.co.nz/brand-images/rnz-international.jpg'],
  name = 'RNZ International',
)

streams = []

streams.append(Track(
  artists = [_rnz_artist],
  album = _national_album,
  name = 'National Radio',
  bitrate = 64,
  track_no = len(streams),
  uri = 'http://radionz-ice.streamguys.com/national_aac64',
))

streams.append(Track(
  artists = [_rnz_artist],
  album = _national_album,
  name = 'National Radio High Quality',
  track_no = len(streams),
  bitrate = 128,
  uri = 'http://radionz-ice.streamguys.com/National_aac128',
))


streams.append(Track(
  artists = [_rnz_artist],
  album = _national_album,
  track_no = len(streams),
  name = 'National Radio Low Quality',
  uri = 'http://radionz-ice.streamguys.com/national',
))


streams.append(Track(
  artists = [_rnz_artist],
  album = _concert_album,
  name = 'Concert FM',
  track_no = len(streams),
  uri = 'http://radionz-ice.streamguys.com/concert_aac64',
))

streams.append(Track(
  artists = [_rnz_artist],
  album = _concert_album,
  name = 'Concert FM High Quality',
  track_no = len(streams),
  uri = 'http://radionz-ice.streamguys.com/Concert_aac128',
))

streams.append(Track(
  artists = [_rnz_artist],
  album = _concert_album,
  name = 'Concert FM Low Quality',
  track_no = len(streams),
  uri = 'http://radionz-ice.streamguys.com/concert',
))


streams.append(Track(
  artists = [_rnz_artist],
  album = _international_album,
  name = 'RNZ International',
  track_no = len(streams),
  uri = 'http://radionz-ice.streamguys.com/international_aac64',
))

streams.append(Track(
  artists = [_rnz_artist],
  album = _parliament_album,
  name = 'Parliament Live Stream',
  track_no = len(streams),
  uri = 'http://radionz-ice.streamguys.com/parliament',
))

news_track = Track(
  artists = [_rnz_artist],
  album = _news_album,
  name = 'Latest News Bulletin',
  track_no = 1,
)

stream_map = {}
for stream in streams:
  stream_map[stream.name] = stream










from __future__ import unicode_literals

import re
from setuptools import find_packages, setup


def get_version(filename):
    with open(filename) as fh:
        metadata = dict(re.findall("__([a-z]+)__ = '([^']+)'", fh.read()))
        return metadata['version']


setup(
    name='Mopidy-RNZ',
    version=get_version('mopidy_rnz/__init__.py'),
    url='https://github.com/danbrough/mopidy-rnz',
    license='Apache License, Version 2.0',
    author='Dan Brough',
    author_email='dan@danbrough.org',
    description='Mopidy extension for RNZ content',
    long_description=open('README.rst').read(),
    packages=find_packages(exclude=['tests', 'tests.*']),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'setuptools',
        'Mopidy >= 1.0',
        'Pykka >= 1.1',
        'requests-cache >= 0.4.13',
        'python_dateutil >= 2.6.1',
    ],
    entry_points={
        'mopidy.ext': [
            'rnz = mopidy_rnz:Extension',
        ],
    },
    classifiers=[
        'Environment :: No Input/Output (Daemon)',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Topic :: Multimedia :: Sound/Audio :: Players',
    ],
)
